#!/usr/bin/env python

import sys
import os
import pwd
import grp

import _pythonpath
_ = _pythonpath

from charmhelpers.contrib.charmsupport import volumes
from charmhelpers.core import hookenv
from charmhelpers.core import host

def storage_is_persistent():
    if os.path.islink(CACHE_DIR):
        target = os.readlink(CACHE_DIR)
        if os.path.ismount(target):
            return True
    return False

def volume_change_pre():
    host.service_stop('retracer-i386')
    host.service_stop('retracer-amd64')
    host.service_stop('retracer-armhf')


def volume_change_post():
    pass


def set_permissions():
    if storage_is_persistent():
        # make sure data on external storage are owned
        # by the jetty user
        user = os.environ.get("USER_CODE_RUNNER")
        group = os.environ.get("GROUP_CODE_RUNNER")
        if user:
            uid = pwd.getpwnam(user).pw_uid
        else:
            uid = os.getuid()
        if group:
            gid = grp.getgrnam(group).gr_gid
        else:
            gid = pwd.getpwuid(uid).pw_gid
        os.chown(CACHE_DIR, uid, gid)
        for root, dirs, files in os.walk(CACHE_DIR):
            for file in files:
                os.chown(os.path.join(root, file), uid, gid)
            for subdir in dirs:
                os.chown(os.path.join(root, subdir), uid, gid)


CACHE_DIR = os.environ['CACHE_DIR']
SAVED_DIR = "{}.{}".format(CACHE_DIR, 'charm_saved')

if __name__ == '__main__':
    try:
        mountpoint = volumes.configure_volume(before_change=volume_change_pre, after_change=volume_change_post)
    except volumes.VolumeConfigurationError:
        hookenv.log('Storage could not be configured', hookenv.ERROR)
        sys.exit(1)

    if mountpoint == 'ephemeral':
        if os.path.islink(CACHE_DIR):
            os.remove(CACHE_DIR)
            if os.path.isdir(SAVED_DIR):
                os.rename(SAVED_DIR, CACHE_DIR)
            else:
                os.path.mkdir(CACHE_DIR)
                set_permissions()
    else:
        if not storage_is_persistent():
            try:
                os.rename(CACHE_DIR, SAVED_DIR)
            except OSError as e:
                hookenv.log('ERROR: could not preserve existing log directory', hookenv.ERROR)
                hookenv.log(e.strerror, hookenv.ERROR)
                sys.exit(1)
            os.symlink(mountpoint, CACHE_DIR)
            set_permissions()
            host.service_start('retracer-i386')
            host.service_start('retracer-amd64')
            host.service_start('retracer-armhf')
        else:
            hookenv.log('%s is already a sym link to persistent storage' % CACHE_DIR, hookenv.INFO)
